"""
Tests for System Mediator package arbitrator
"""
import logging
import sys
from typing import List
from unittest.mock import patch, MagicMock

# pylint: disable=E0401, R0201, W0621
import argparse
import pytest

from system_mediator.arbitrator import Arbiter

from testing.utils.conftest import mock_workflow_info, mock_capability_info

from workspaces.capability.schema import CapabilityExecution

from workspaces.capability.services.capability_info import CapabilityInfo
from workspaces.workflow.schema import WorkflowRequest
from workspaces.workflow.services.workflow_info import WorkflowInfo

pytest_plugins = ["testing.utils.conftest"]

logger = logging.Logger("test_arbitrator")
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))

nsp = argparse.Namespace(force_fail=["-1"])


@pytest.mark.skip("Broken due to queue/messenger rework")
@patch("system_mediator.arbitrator.MessageSender", MagicMock())
@pytest.mark.usefixtures("mock_workflow_info", "mock_capability_info")
def mock_arbiter(mock_workflow_info: WorkflowInfo, mock_capability_info: CapabilityInfo) -> Arbiter:
    """
    A mock arbiter for our tests.

    :param mock_workflow_info: fake workflow info
    :param mock_capability_info: fake capability info

    :return:
    """
    return Arbiter(mock_workflow_info, mock_capability_info, nsp)


@pytest.mark.usefixtures("mock_workflow_requests")
def mock_message(mock_workflow_requests: List[WorkflowRequest]):
    """
    A mock message for our tests.

    :param mock_workflow_requests: a bunch of fake workflow requests

    :return:
    """
    failed_msg = {
        "service": "workflow",
        "routing_key": "workflow",
        "subject": mock_workflow_requests[0].__json__(),
        "type": "workflow-failed",
    }
    return failed_msg


arbiter = mock_arbiter(mock_workflow_info=mock_workflow_info, mock_capability_info=mock_capability_info)


@pytest.mark.usefixtures(
    "mock_workflow_info",
    "mock_capability_info",
    "mock_workflow_requests",
    "mock_capability_execution",
    "mock_capability_requests",
)
class TestArbiter:
    """Tests for the arbitrator"""

    @pytest.mark.skip("Broken due to queue/messenger rework")
    def test_which_option(self):
        """
        Confirm that which_option works as expected.

        :return:
        """
        arbiter.which_option(nsp)
        assert arbiter.intervention == "force_fail"
        assert arbiter.wf_request_id == "-1"

    @pytest.mark.skip("ignores router mock?")
    @pytest.mark.usefixtures("mock_workflow_requests")
    def test_intervene_fail(
        self,
        mock_workflow_info: WorkflowInfo,
        mock_workflow_requests: List[WorkflowRequest],
    ):
        """
        Make sure failure message is sent.

        :param mock_workflow_info: stand-in for workflow info
        :param mock_workflow_requests: stand-in for workflow requests

        :return:
        """
        with patch("messaging.router.Router") as mock_router:
            mock_workflow_info.lookup_workflow_definition = MagicMock(return_value=mock_workflow_requests[0])
            arbiter.info = mock_workflow_info
            arbiter.message_router = mock_router
            arbiter.intervene_fail(arbiter.wf_request_id)
            assert arbiter.message_router.send_message.call_count == 1

    @pytest.mark.skip("Broken due to queue/messenger rework")
    @pytest.mark.usefixtures("mock_workflow_requests", "mock_capability_execution")
    def test_resolve_fail(
        self,
        mock_workflow_requests: List[WorkflowRequest],
        mock_capability_execution: CapabilityExecution,
    ):
        """
        Make sure failure is resolved.

        :param mock_workflow_requests:
        :param mock_capability_execution:
        :return:
        """
        arbiter.info.lookup_workflow_request = MagicMock(return_value=mock_workflow_requests[0])
        arbiter.cinfo.lookup_execution_by_workflow_request_id = MagicMock(return_value=mock_capability_execution)
        assert arbiter.resolve_fail(arbiter.wf_request_id) is None

    @pytest.mark.skip("Broken due to queue/messenger rework")
    @pytest.mark.usefixtures("mock_workflow_requests", "mock_capability_execution")
    def test_check_status(
        self,
        mock_capability_info: CapabilityInfo,
        mock_workflow_requests: List[WorkflowRequest],
        mock_capability_execution: CapabilityExecution,
    ):
        """

        Confirm that check_status() works as expected.

        :param mock_capability_info:
        :param mock_workflow_requests:
        :param mock_capability_execution:
        :return:
        """
        arbiter.cinfo = mock_capability_info
        arbiter.info.lookup_workflow_request = MagicMock(return_value=mock_workflow_requests[0])
        arbiter.cinfo.lookup_execution_by_workflow_request_id = MagicMock(return_value=mock_capability_execution)

        assert arbiter.check_status(arbiter.wf_request_id) is True

    @pytest.mark.skip("Broken due to queue/messenger rework")
    def test_intercede(self):
        assert arbiter.intercede() is None
