from datetime import datetime

from workspaces.workflow.schema import WorkflowRequest

from workspaces.system.views import ViewWorkflowCapoParameters
from immutable_views import DictView


"""
tests for views.ViewWorkflowCapoParameters
"""
expected_list = DictView(
    {
        "general": [
            "edu.nrao.workspaces.WorkflowSettings.serviceUrl",
            "edu.nrao.workspaces.NotificationSettings.serviceUrl",
            "edu.nrao.workspaces.UISettings.serviceUrl",
        ],
        "casa_requirements": [
            "edu.nrao.workspaces.ProcessingSettings.useCasa",
            "edu.nrao.archive.workflow.config.CasaVersions.homeForReprocessing",
        ],
    }
)

request = WorkflowRequest(
    workflow_request_id=-3,
    workflow_name="vlass_seci",
    argument={},
    state="Created",
    created_at=datetime(2021, 1, 1),
    updated_at=datetime(2021, 1, 1),
)
view = ViewWorkflowCapoParameters(request)


class TestViewWorkflowCapoParameters:
    def test_general_parameters(self):
        general_params = view.general_parameters()
        assert general_params == expected_list["general"]

    def test_casa_parameters(self):
        casa_params = view.casa_parameters()
        assert casa_params == expected_list["casa_requirements"]

    def test_construct_view(self):
        constructed_view = view.construct_view()
        assert constructed_view == [
            "edu.nrao.workspaces.WorkflowSettings.serviceUrl",
            "edu.nrao.workspaces.NotificationSettings.serviceUrl",
            "edu.nrao.workspaces.UISettings.serviceUrl",
            "edu.nrao.workspaces.ProcessingSettings.useCasa",
            "edu.nrao.archive.workflow.config.CasaVersions.homeForReprocessing",
        ]
